@@ -35,12 +35,6 @@ extern char key_lines[NUM_KEY_LINES][MAXCMDLINE];
 extern int edit_line;
 extern int key_linepos;
 
-void
-DrawString(int x, int y, char *s)
-{
-	DrawStringScaled(x, y, s, 1.0f);
-}
-
 void
 DrawStringScaled(int x, int y, char *s, float factor)
 {
@@ -52,12 +46,6 @@ DrawStringScaled(int x, int y, char *s, float factor)
 	}
 }
 
-void
-DrawAltString(int x, int y, char *s)
-{
-	DrawAltStringScaled(x, y, s, 1.0f);
-}
-
 void
 DrawAltStringScaled(int x, int y, char *s, float factor)
 {
@@ -108,6 +96,13 @@ Con_ToggleConsole_f(void)
 	{
 		M_ForceMenuOff();
 		Cvar_Set("paused", "0");
+
+		/* play music */
+		if (Cvar_VariableValue("ogg_pausewithgame") == 1 &&
+		    OGG_Status() == PAUSE)
+		{
+		    Cbuf_AddText("ogg toggle\n");
+		}
 	}
 	else
 	{
@@ -118,6 +113,13 @@ Con_ToggleConsole_f(void)
 			Com_ServerState())
 		{
 			Cvar_Set("paused", "1");
+
+			/* pause music */
+			if (Cvar_VariableValue("ogg_pausewithgame") == 1 &&
+			    OGG_Status() == PLAY)
+			{
+			    Cbuf_AddText("ogg toggle\n");
+			}
 		}
 	}
 }
@@ -177,7 +179,7 @@ Con_Dump_f(void)
 
 	Com_Printf("Dumped console text to %s.\n", name);
 	FS_CreatePath(name);
-	f = fopen(name, "w");
+	f = Q_fopen(name, "w");
 
 	if (!f)
 	{
@@ -271,7 +273,14 @@ Con_CheckResize(void)
 	char tbuf[CON_TEXTSIZE];
 	float scale = SCR_GetConsoleScale();
 
-	width = ((int)(viddef.width / scale) >> 3) - 2;
+	/* We need to clamp the line width to MAXCMDLINE - 2,
+	   otherwise we may overflow the text buffer if the
+	   vertical resultion / 8 (one char == 8 pixels) is
+	   bigger then MAXCMDLINE.
+	   MAXCMDLINE - 2 because 1 for the prompt and 1 for
+	   the terminating \0. */
+	width = ((int)(viddef.width / scale) / 8) - 2;
+	width = width > MAXCMDLINE - 2 ? MAXCMDLINE - 2 : width;
 
 	if (width == con.linewidth)
 	{
@@ -452,29 +461,6 @@ Con_Print(char *txt)
 	}
 }
 
-void
-Con_CenteredPrint(char *text)
-{
-	int l;
-	char buffer[1024];
-
-	l = strlen(text);
-	l = (con.linewidth - l) / 2;
-
-	if (l <= 0)
-	{
-		l = 0;
-	}
-	else
-	{
-		memset(buffer, ' ', l);
-	}
-
-	strcpy(buffer + l, text);
-	strcat(buffer, "\n");
-	Con_Print(buffer);
-}
-
 /*
  * The input line scrolls horizontally if
  * typing goes beyond the right edge
@@ -600,7 +586,7 @@ Con_DrawNotify(void)
 			x++;
 		}
 
-		Draw_CharScaled(((x + skip) << 3) * scale, v + scale, 10 + ((cls.realtime >> 8) & 1), scale);
+		Draw_CharScaled(((x + skip) << 3) * scale, v * scale, 10 + ((cls.realtime >> 8) & 1), scale);
 		v += 8;
 	}
 
@@ -713,7 +699,11 @@ Con_DrawConsole(float frac)
 	}
 
 	/* draw the download bar, figure out width */
+#ifdef USE_CURL
+	if (cls.downloadname[0] && (cls.download || cls.downloadposition))
+#else
 	if (cls.download)
+#endif
 	{
 		if ((text = strrchr(cls.downloadname, '/')) != NULL)
 		{
