@@ -29,7 +29,6 @@
 
 void CL_ParseStatusMessage(void);
 
-extern cvar_t *msg;
 extern cvar_t *rcon_client_password;
 extern cvar_t *rcon_address;
 extern cvar_t *cl_timeout;
@@ -189,7 +188,7 @@ CL_CheckForResend(void)
 void
 CL_Connect_f(void)
 {
-	char *server;
+	char server[256];
 
 	if (Cmd_Argc() != 2)
 	{
@@ -197,6 +196,8 @@ CL_Connect_f(void)
 		return;
 	}
 
+	Q_strlcpy(server, Cmd_Argv(1), sizeof(server));
+
 	if (Com_ServerState())
 	{
 		/* if running a local server, kill it and reissue
@@ -204,15 +205,7 @@ CL_Connect_f(void)
 		SV_Shutdown("Server quit\n", false);
 	}
 
-	else
-	{
-		CL_Disconnect();
-	}
-
-	server = Cmd_Argv(1);
-
 	NET_Config(true); /* allow remote */
-
 	CL_Disconnect();
 
 	cls.state = ca_connecting;
@@ -285,6 +278,8 @@ CL_Rcon_f(void)
 	NET_SendPacket(NS_CLIENT, strlen(message) + 1, message, to);
 }
 
+void CL_WriteConfiguration(void);
+
 /*
  * Goes from a connected state to full screen
  * console state Sends a disconnect message to
@@ -324,12 +319,8 @@ CL_Disconnect(void)
 	cls.connect_time = 0;
 
 	SCR_StopCinematic();
-#ifdef OGG
+
 	OGG_Stop();
-#endif
-#ifdef CDA
-	CDAudio_Stop();
-#endif
 
 	if (cls.demorecording)
 	{
@@ -354,9 +345,22 @@ CL_Disconnect(void)
 		cls.download = NULL;
 	}
 
+#ifdef USE_CURL
+	CL_CancelHTTPDownloads(true);
+	cls.downloadReferer[0] = 0;
+	cls.downloadname[0] = 0;
+	cls.downloadposition = 0;
+#endif
+
 	cls.state = ca_disconnected;
 
 	snd_is_underwater = false;
+
+	// save config for old game/mod
+	CL_WriteConfiguration();
+
+	// we disconnected, so revert to default game/mod (might have been different mod on MP server)
+	Cvar_Set("game", userGivenGame);
 }
 
 void
@@ -439,10 +443,15 @@ CL_Changing_f(void)
 	}
 
 	SCR_BeginLoadingPlaque();
-
 	cls.state = ca_connected; /* not active anymore, but not disconnected */
-
 	Com_Printf("\nChanging map...\n");
+
+#ifdef USE_CURL
+	if (cls.downloadServerRetry[0] != 0)
+	{
+		CL_SetHTTPServer(cls.downloadServerRetry);
+	}
+#endif
 }
 
 /*
@@ -494,10 +503,11 @@ CL_PingServers_f(void)
 	int i;
 	netadr_t adr;
 	char name[32];
-	char *adrstring;
+	const char *adrstring;
 	cvar_t *noudp;
 	cvar_t *noipx;
 
+	memset(&adr, 0, sizeof(adr));
 	NET_Config(true);  /* allow remote but do we even need lokal pings? */
 
 	/* send a broadcast packet */
@@ -530,7 +540,7 @@ CL_PingServers_f(void)
 	for (i = 0; i < 16; i++)
 	{
 		Com_sprintf(name, sizeof(name), "adr%i", i);
-		adrstring = (char *)Cvar_VariableString(name);
+		adrstring = Cvar_VariableString(name);
 
 		if (!adrstring || !adrstring[0])
 		{
@@ -584,6 +594,64 @@ CL_ConnectionlessPacket(void)
 		}
 
 		Netchan_Setup(NS_CLIENT, &cls.netchan, net_from, cls.quakePort);
+		char *buff = NET_AdrToString(cls.netchan.remote_address);
+
+		for(int i = 1; i < Cmd_Argc(); i++)
+		{
+			char *p = Cmd_Argv(i);
+
+			if(!strncmp(p, "dlserver=", 9))
+			{
+#ifdef USE_CURL
+				p += 9;
+				Com_sprintf(cls.downloadReferer, sizeof(cls.downloadReferer), "quake2://%s", buff);
+				CL_SetHTTPServer (p);
+
+				if (cls.downloadServer[0])
+				{
+					Com_Printf("HTTP downloading enabled, URL: %s\n", cls.downloadServer);
+				}
+#else
+				Com_Printf("HTTP downloading supported by server but not the client.\n");
+#endif
+			}
+		}
+
+		/* Put client into pause mode when connecting to a local server.
+		   This prevents the world from being forwarded while the client
+		   is connecting, loading assets, etc. It's not 100%, there're
+		   still 4 world frames (for baseq2) processed in the game and
+		   100 frames by the server if the player enters a level that he
+		   or she already visited. In practise both shouldn't be a big
+		   problem. 4 frames are hardly enough for monsters staring to
+		   attack and in most levels the starting area in unreachable by
+		   monsters and free from environmental effects.
+
+		   Com_Serverstate() returns 2 if the server is local and we're
+		   running a real game and no timedemo, cinematic, etc. The 2 is
+		   taken from the server_state_t enum value 'ss_game'. If it's a
+		   local server, maxclients aus either 0 (for single player), or
+		   2 to 8 (coop and deathmatch) if we're reaching this code.
+		   For remote servers it's always 1. So this should trigger only
+		   if it's a local single player server.
+
+		   Since the player can load savegames from a paused state (e.g.
+		   through the console) we'll need to communicate if we entered
+		   paused mode (and it should left as soon as the player joined
+		   the server) or if it was already there.
+
+		   Last but not least this can be disabled by cl_loadpaused 0. */
+		if (Com_ServerState() == 2 && (Cvar_VariableValue("maxclients") <= 1))
+		{
+			if (cl_loadpaused->value)
+			{
+				if (!cl_paused->value)
+				{
+					paused_at_load = true;
+					Cvar_Set("paused", "1");
+				}
+			}
+		}
 
 		MSG_WriteChar(&cls.netchan.message, clc_stringcmd);
 		MSG_WriteString(&cls.netchan.message, "new");
