@@ -492,10 +492,20 @@ CL_ParseLaser(int colors)
 	{
 		if (l->endtime < cl.time)
 		{
+			float alpha = cl_laseralpha->value;
+			if (alpha < 0.0f)
+			{
+				alpha = 0.0f;
+			}
+			else if (alpha > 1.0f)
+			{
+				alpha = 1.0f;
+			}
+
 			l->ent.flags = RF_TRANSLUCENT | RF_BEAM;
 			VectorCopy(start, l->ent.origin);
 			VectorCopy(end, l->ent.oldorigin);
-			l->ent.alpha = 0.30f;
+			l->ent.alpha = alpha;
 			l->ent.skinnum = (colors >> ((randk() % 4) * 8)) & 0xff;
 			l->ent.model = NULL;
 			l->ent.frame = 4;
