@@ -25,17 +25,24 @@
  */
 
 #include "header/client.h"
+#include "input/header/input.h"
 
 /* development tools for weapons */
 int gun_frame;
 struct model_s *gun_model;
 
 cvar_t *crosshair;
+cvar_t *crosshair_3d;
+cvar_t *crosshair_3d_glow;
+
 cvar_t *crosshair_scale;
 cvar_t *cl_testparticles;
 cvar_t *cl_testentities;
 cvar_t *cl_testlights;
 cvar_t *cl_testblend;
+cvar_t *crosshair_3d_glow_r;
+cvar_t *crosshair_3d_glow_g;
+cvar_t *crosshair_3d_glow_b;
 
 cvar_t *cl_stats;
 
@@ -53,6 +60,8 @@ lightstyle_t r_lightstyles[MAX_LIGHTSTYLES];
 char cl_weaponmodels[MAX_CLIENTWEAPONMODELS][MAX_QPATH];
 int num_cl_weaponmodels;
 
+void V_Render3dCrosshair(void);
+
 /*
  * Specifies the model that will be used as the world
  */
@@ -174,8 +183,8 @@ V_TestEntities(void)
 	{
 		ent = &r_entities[i];
 
-		r = 64 * ((i % 4) - 1.5);
-		f = 64 * (i / 4) + 128;
+		r = 64.0f * ((float)(i % 4) - 1.5f);
+		f = (float)(64 * (i / 4) + 128);
 
 		for (j = 0; j < 3; j++)
 		{
@@ -227,7 +236,7 @@ V_TestLights(void)
 void
 CL_PrepRefresh(void)
 {
-	char mapname[32];
+	char mapname[MAX_QPATH];
 	int i;
 	char name[MAX_QPATH];
 	float rotate;
@@ -273,7 +282,7 @@ CL_PrepRefresh(void)
 		}
 
 		SCR_UpdateScreen();
-		Sys_SendKeyEvents();
+		IN_Update();
 
 		if (name[0] == '#')
 		{
@@ -313,7 +322,7 @@ CL_PrepRefresh(void)
 	for (i = 1; i < MAX_IMAGES && cl.configstrings[CS_IMAGES + i][0]; i++)
 	{
 		cl.image_precache[i] = Draw_FindPic(cl.configstrings[CS_IMAGES + i]);
-		Sys_SendKeyEvents();
+		IN_Update();
 	}
 
 	Com_Printf("                                     \r");
@@ -327,7 +336,7 @@ CL_PrepRefresh(void)
 
 		Com_Printf("client %i\r", i);
 		SCR_UpdateScreen();
-		Sys_SendKeyEvents();
+		IN_Update();
 		CL_ParseClientinfo(i);
 		Com_Printf("                                     \r");
 	}
@@ -352,38 +361,10 @@ CL_PrepRefresh(void)
 	cl.refresh_prepped = true;
 	cl.force_refdef = true; /* make sure we have a valid refdef */
 
-#if defined(OGG) || defined(CDA)
-
 	/* start the cd track */
-	if (Cvar_VariableValue("cd_shuffle"))
-	{
- #ifdef CDA
-		CDAudio_RandomPlay();
- #endif
-	}
-	else
-	{
- #ifdef CDA
-		CDAudio_Play((int)strtol(cl.configstrings[CS_CDTRACK], (char **)NULL, 10), true);
- #endif
+	int track = (int)strtol(cl.configstrings[CS_CDTRACK], (char **)NULL, 10);
 
- #ifdef OGG
-
-		/* OGG/Vorbis */
-		if ((int)strtol(cl.configstrings[CS_CDTRACK], (char **)NULL, 10) < 10)
-		{
-			char tmp[3] = "0";
-			OGG_ParseCmd(strcat(tmp, cl.configstrings[CS_CDTRACK]));
-		}
-		else
-		{
-			OGG_ParseCmd(cl.configstrings[CS_CDTRACK]);
-		}
-
- #endif
-	}
-
-#endif
+	OGG_PlayTrack(track, true, true);
 }
 
 float
@@ -453,8 +434,7 @@ entitycmpfnc(const entity_t *a, const entity_t *b)
 	}
 	else
 	{
- 		return (a->model == b->model) ? 0 :
-			(a->model > b->model) ? 1 : -1;
+		return (a->model > b->model) ? 1 : -1;
 	}
 }
 
@@ -463,12 +443,14 @@ V_RenderView(float stereo_separation)
 {
 	if (cls.state != ca_active)
 	{
+		R_EndWorldRenderpass();
 		return;
 	}
 
 	if (!cl.refresh_prepped)
 	{
-		return;
+		R_EndWorldRenderpass();
+		return;			// still loading
 	}
 
 	if (cl_timedemo->value)
@@ -494,6 +476,9 @@ V_RenderView(float stereo_separation)
 		   v_forward, etc. */
 		CL_AddEntities();
 
+		// before changing viewport we should trace the crosshair position
+		V_Render3dCrosshair();
+
 		if (cl_testparticles->value)
 		{
 			V_TestParticles();
@@ -519,6 +504,7 @@ V_RenderView(float stereo_separation)
 
 		/* offset vieworg appropriately if
 		   we're doing stereo separation */
+
 		if (stereo_separation != 0)
 		{
 			vec3_t tmp;
@@ -572,6 +558,18 @@ V_RenderView(float stereo_separation)
 		qsort(cl.refdef.entities, cl.refdef.num_entities,
 				sizeof(cl.refdef.entities[0]), (int (*)(const void *, const void *))
 				entitycmpfnc);
+	} else if (cl.frame.valid && cl_paused->value && gl1_stereo->value) {
+		// We need to adjust the refdef in stereo mode when paused.
+		vec3_t tmp;
+		CL_CalcViewValues();
+		VectorScale( cl.v_right, stereo_separation, tmp );
+		VectorAdd( cl.refdef.vieworg, tmp, cl.refdef.vieworg );
+
+		cl.refdef.vieworg[0] += 1.0/16;
+		cl.refdef.vieworg[1] += 1.0/16;
+		cl.refdef.vieworg[2] += 1.0/16;
+
+		cl.refdef.time = cl.time*0.001;
 	}
 
 	cl.refdef.x = scr_vrect.x;
@@ -602,12 +600,60 @@ V_RenderView(float stereo_separation)
 	SCR_DrawCrosshair();
 }
 
+void
+V_Render3dCrosshair(void)
+{
+	trace_t crosshair_trace;
+	vec3_t end;
+
+	crosshair_3d = Cvar_Get("crosshair_3d", "0", CVAR_ARCHIVE);
+	crosshair_3d_glow = Cvar_Get("crosshair_3d_glow", "0", CVAR_ARCHIVE);
+
+
+	if(crosshair_3d->value || crosshair_3d_glow->value){
+		VectorMA(cl.refdef.vieworg,8192,cl.v_forward,end);
+		crosshair_trace = CL_PMTrace(cl.refdef.vieworg, vec3_origin, vec3_origin, end);
+
+		if(crosshair_3d_glow->value){
+			crosshair_3d_glow_r = Cvar_Get("crosshair_3d_glow_r", "5", CVAR_ARCHIVE);
+			crosshair_3d_glow_g = Cvar_Get("crosshair_3d_glow_g", "1", CVAR_ARCHIVE);
+			crosshair_3d_glow_b = Cvar_Get("crosshair_3d_glow_b", "4", CVAR_ARCHIVE);
+
+			V_AddLight(
+				crosshair_trace.endpos,
+				crosshair_3d_glow->value,
+				crosshair_3d_glow_r->value,
+				crosshair_3d_glow_g->value,
+				crosshair_3d_glow_b->value
+			);
+		}
+
+		if(crosshair_3d->value){
+			entity_t crosshair_ent = {0};
+
+			crosshair_ent.origin[0] = crosshair_trace.endpos[0];
+			crosshair_ent.origin[1] = crosshair_trace.endpos[1];
+			crosshair_ent.origin[2] = crosshair_trace.endpos[2];
+
+			crosshair_ent.model = R_RegisterModel("models/crosshair/tris.md2");
+			//crosshair_ent.skin = R_RegisterSkin("models/crosshair/skin.pcx");
+
+			AngleVectors2(crosshair_trace.plane.normal, crosshair_ent.angles);
+			crosshair_ent.flags = RF_DEPTHHACK | RF_FULLBRIGHT | RF_NOSHADOW;
+
+			V_AddEntity(&crosshair_ent);
+		}
+	}
+}
+
 void
 V_Viewpos_f(void)
 {
-	Com_Printf("(%i %i %i) : %i\n", (int)cl.refdef.vieworg[0],
-			(int)cl.refdef.vieworg[1], (int)cl.refdef.vieworg[2],
-			(int)cl.refdef.viewangles[YAW]);
+	Com_Printf("position: %i %i %i, angles: %i %i %i\n",
+			(int)cl.refdef.vieworg[0], (int)cl.refdef.vieworg[1],
+			(int)cl.refdef.vieworg[2],
+			(int)cl.refdef.viewangles[PITCH], (int)cl.refdef.viewangles[YAW],
+			(int)cl.refdef.viewangles[ROLL]);
 }
 
 void
@@ -620,7 +666,7 @@ V_Init(void)
 	Cmd_AddCommand("viewpos", V_Viewpos_f);
 
 	crosshair = Cvar_Get("crosshair", "0", CVAR_ARCHIVE);
-	crosshair_scale = Cvar_Get("crosshair_scale", "1", CVAR_ARCHIVE);
+	crosshair_scale = Cvar_Get("crosshair_scale", "-1", CVAR_ARCHIVE);
 	cl_testblend = Cvar_Get("cl_testblend", "0", 0);
 	cl_testparticles = Cvar_Get("cl_testparticles", "0", 0);
 	cl_testentities = Cvar_Get("cl_testentities", "0", 0);
