@@ -68,13 +68,14 @@ static menulist_s s_fs_box;
 static menulist_s s_vsync_list;
 static menulist_s s_af_list;
 static menulist_s s_msaa_list;
+static menulist_s s_filter_list;
 static menuaction_s s_defaults_action;
 static menuaction_s s_apply_action;
 
 // --------
 
-// gl1, gl3, gles3, vk, soft
-#define MAXRENDERERS 5
+// gl1, gl3, gles3, gl4, vk, soft
+#define MAXRENDERERS 6
 
 typedef struct
 {
@@ -111,6 +112,13 @@ Renderer_FillRenderdef(void)
 		rendererlist[numrenderer].cvarstr = "gles3";
 	}
 
+	if (VID_HasRenderer("gl4"))
+	{
+		numrenderer++;
+		rendererlist[numrenderer].boxstr = "[OpenGL 4.6]";
+		rendererlist[numrenderer].cvarstr = "gl4";
+	}
+
 	if (VID_HasRenderer("vk"))
 	{
 		numrenderer++;
@@ -178,6 +186,39 @@ ResetDefaults(void *unused)
 #define CUSTOM_MODE_NAME "[Custom    ]"
 #define AUTO_MODE_NAME   "[Auto      ]"
 
+static void
+ApplyFilter(void* unused)
+{
+	if (Q_stricmp(vid_renderer->string, "gl3") == 0 || Q_stricmp(vid_renderer->string, "gles3") == 0 ||
+		Q_stricmp(vid_renderer->string, "gl1") == 0)
+	{
+		if (s_filter_list.curvalue == 0)
+		{
+			Cvar_Set("gl_texturemode", "GL_NEAREST");
+		}
+		else if (s_filter_list.curvalue == 1)
+		{
+			Cvar_Set("gl_texturemode", "GL_LINEAR_MIPMAP_NEAREST");
+		}
+		else if (s_filter_list.curvalue == 2)
+		{
+			Cvar_Set("gl_texturemode", "GL_LINEAR_MIPMAP_LINEAR");
+		}
+	}
+
+	if (Q_stricmp(vid_renderer->string, "soft") == 0)
+	{
+		if (s_filter_list.curvalue == 0)
+		{
+			Cvar_Set("sw_texture_filtering", "0");
+		}
+		else if (s_filter_list.curvalue == 1)
+		{
+			Cvar_Set("sw_texture_filtering", "1");
+		}
+	}
+}
+
 static void
 ApplyChanges(void *unused)
 {
@@ -379,6 +420,12 @@ VID_MenuInit(void)
 		0
 	};
 
+	static const char *onoff_names[] = {
+		"off",
+		"on",
+		0
+	};
+	
 	static const char *yesno_names[] = {
 		"no",
 		"yes",
@@ -401,6 +448,14 @@ VID_MenuInit(void)
 		0
 	};
 
+	static const char *filter_names[] = {
+		"pixelated",
+		"standard",
+		"trilinear",
+		"custom",
+		0
+	};
+
 	if (!r_mode)
 	{
 		r_mode = Cvar_Get("r_mode", "4", 0);
@@ -582,7 +637,7 @@ VID_MenuInit(void)
 		s_gl1_overbrightbits_slider.generic.y = (y += 10);
 		s_gl1_overbrightbits_slider.cvar = "gl1_overbrightbits";
 		s_gl1_overbrightbits_slider.minvalue = 0;
-		s_gl1_overbrightbits_slider.maxvalue = 3;
+		s_gl1_overbrightbits_slider.maxvalue = 2;
 		s_gl1_overbrightbits_slider.slidestep = 1;
 		s_gl1_overbrightbits_slider.printformat = "%.0f";
 
@@ -666,6 +721,54 @@ VID_MenuInit(void)
 				pow(2, s_msaa_list.curvalue) <= gl_msaa_samples->value);
 		s_msaa_list.curvalue--;
 	}
+	
+	s_filter_list.generic.type = MTYPE_SPINCONTROL;
+	s_filter_list.generic.name = "texture filter";
+	s_filter_list.curvalue = 0;
+	s_filter_list.generic.callback = ApplyFilter;
+
+	const char* filter = NULL;
+	int mode = 0;
+	
+	if (Q_stricmp(vid_renderer->string, "gl3") == 0 || Q_stricmp(vid_renderer->string, "gles3") == 0 ||
+		Q_stricmp(vid_renderer->string, "gl1") == 0)
+	{	
+		s_filter_list.generic.x = 0;
+		s_filter_list.generic.y = (y += 10);
+		s_filter_list.itemnames = filter_names;
+		
+		filter = Cvar_VariableString("gl_texturemode");
+		mode = 3;
+
+		if (Q_stricmp(filter, "GL_NEAREST") == 0)
+		{
+			mode = 0;
+		}
+		else if (Q_stricmp(filter, "GL_LINEAR_MIPMAP_NEAREST") == 0)
+		{
+			mode = 1;
+		}
+		else if (Q_stricmp(filter, "GL_LINEAR_MIPMAP_LINEAR") == 0)
+		{
+			mode = 2;
+		}
+	}
+	else if (Q_stricmp(vid_renderer->string, "soft") == 0)
+	{
+		s_filter_list.generic.x = 0;
+		s_filter_list.generic.y = (y += 10);
+		s_filter_list.itemnames = onoff_names;
+
+		filter = Cvar_VariableString("sw_texture_filtering");
+		mode = 0;
+
+		if (Q_stricmp(filter, "1") == 0)
+		{
+			mode = 1;
+		}	
+	}
+
+	s_filter_list.curvalue = mode;
 
 	s_defaults_action.generic.type = MTYPE_ACTION;
 	s_defaults_action.generic.name = "reset to default";
@@ -713,6 +816,11 @@ VID_MenuInit(void)
 	Menu_AddItem(&s_opengl_menu, (void *)&s_vsync_list);
 	Menu_AddItem(&s_opengl_menu, (void *)&s_af_list);
 	Menu_AddItem(&s_opengl_menu, (void *)&s_msaa_list);
+	if (Q_stricmp(vid_renderer->string, "gl3") == 0 || Q_stricmp(vid_renderer->string, "gles3") == 0 ||
+		Q_stricmp(vid_renderer->string, "gl1") == 0 || Q_stricmp(vid_renderer->string, "soft") == 0)
+	{
+		Menu_AddItem(&s_opengl_menu, (void *)&s_filter_list);
+	}
 	Menu_AddItem(&s_opengl_menu, (void *)&s_defaults_action);
 	Menu_AddItem(&s_opengl_menu, (void *)&s_apply_action);
 
@@ -774,3 +882,18 @@ VID_MenuKey(int key)
 	return sound;
 }
 
+/*
+ * VIDEO MENU
+ */
+
+void
+M_Menu_Video_f(void)
+{
+    VID_MenuInit();
+    s_opengl_menu.draw = VID_MenuDraw;
+    s_opengl_menu.key  = VID_MenuKey;
+
+    M_PushMenu(&s_opengl_menu);
+}
+
+
