@@ -31,7 +31,7 @@ int numgltextures;
 static int image_max = 0;
 int base_textureid; /* gltextures[i] = base_textureid+i */
 extern qboolean scrap_dirty;
-extern byte scrap_texels[MAX_SCRAPS][BLOCK_WIDTH * BLOCK_HEIGHT];
+extern byte *scrap_texels[MAX_SCRAPS];
 
 static byte intensitytable[256];
 static unsigned char gammatable[256];
@@ -144,6 +144,21 @@ R_SetTexturePalette(unsigned palette[256])
 	}
 }
 
+void
+R_SelectTexture(GLenum texture)
+{
+	if (!gl_config.multitexture || gl_state.currenttarget == texture)
+	{
+		return;
+	}
+
+	gl_state.currenttmu = texture - GL_TEXTURE0;
+	gl_state.currenttarget = texture;
+
+	qglActiveTexture(texture);
+	qglClientActiveTexture(texture);
+}
+
 void
 R_TexEnv(GLenum mode)
 {
@@ -151,12 +166,12 @@ R_TexEnv(GLenum mode)
 
 	if (mode != lastmodes[gl_state.currenttmu])
 	{
-		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, mode); // FIXME: shouldn't this be glTexEnvi() ?
+		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, mode);
 		lastmodes[gl_state.currenttmu] = mode;
 	}
 }
 
-void
+qboolean
 R_Bind(int texnum)
 {
 	extern image_t *draw_chars;
@@ -168,11 +183,66 @@ R_Bind(int texnum)
 
 	if (gl_state.currenttextures[gl_state.currenttmu] == texnum)
 	{
-		return;
+		return false;
 	}
 
 	gl_state.currenttextures[gl_state.currenttmu] = texnum;
 	glBindTexture(GL_TEXTURE_2D, texnum);
+	return true;
+}
+
+void
+R_MBind(GLenum target, int texnum)
+{
+	const int tmu = target - GL_TEXTURE0;
+
+	if (target != gl_state.currenttarget)
+	{
+		R_SelectTexture(target);
+	}
+
+	if (gl_state.currenttextures[tmu] == texnum)
+	{
+		return;
+	}
+
+	R_Bind(texnum);
+}
+
+void
+R_EnableMultitexture(qboolean enable)
+{
+	static qboolean active;
+
+	if (!gl_config.multitexture || enable == active)
+	{
+		return;	// current state is the right one
+	}
+
+	active = enable;
+	R_SelectTexture(GL_TEXTURE1);
+
+	if (active && !r_fullbright->value)
+	{
+		glEnable(GL_TEXTURE_2D);
+
+		if (gl_lightmap->value)
+		{
+			R_TexEnv(GL_REPLACE);
+		}
+		else
+		{
+			R_TexEnv(GL_MODULATE);
+		}
+	}
+	else	// disable multitexturing
+	{
+		glDisable(GL_TEXTURE_2D);
+		R_TexEnv(GL_REPLACE);
+	}
+
+	R_SelectTexture(GL_TEXTURE0);
+	R_TexEnv(GL_REPLACE);
 }
 
 void
@@ -230,7 +300,11 @@ R_TextureMode(char *string)
 			nolerp = true;
 		}
 
-		R_Bind(glt->texnum);
+		if ( !R_Bind(glt->texnum) )
+		{
+			continue;	// don't bother changing anything if texture was already set
+		}
+
 		if ((glt->type != it_pic) && (glt->type != it_sky)) /* mipmapped texture */
 		{
 			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_min);
@@ -240,7 +314,7 @@ R_TextureMode(char *string)
 			if (gl_config.anisotropic && gl_anisotropic->value)
 			{
 				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT,
-						max(gl_anisotropic->value, 1.f));
+						Q_max(gl_anisotropic->value, 1.f));
 			}
 		}
 		else /* texture has no mipmaps */
@@ -527,11 +601,6 @@ R_BuildPalettedTexture(unsigned char *paletted_texture, unsigned char *scaled,
 	}
 }
 
-// Windows headers don't define this constant.
-#ifndef GL_GENERATE_MIPMAP
-#define GL_GENERATE_MIPMAP 0x8191
-#endif
-
 qboolean
 R_Upload32Native(unsigned *data, int width, int height, qboolean mipmap)
 {
@@ -559,11 +628,11 @@ R_Upload32Native(unsigned *data, int width, int height, qboolean mipmap)
 			break;
 		}
 	}
-    glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, mipmap);
+	glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, mipmap);
 	glTexImage2D(GL_TEXTURE_2D, 0, comp, width,
 			height, 0, GL_RGBA, GL_UNSIGNED_BYTE,
 			data);
-    glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, false);
+	glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, false);
 	return samples == gl_alpha_format;
 }
 
@@ -778,7 +847,7 @@ R_Upload32(unsigned *data, int width, int height, qboolean mipmap)
 	if (mipmap && gl_config.anisotropic && gl_anisotropic->value)
 	{
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT,
-				max(gl_anisotropic->value, 1.f));
+				Q_max(gl_anisotropic->value, 1.f));
 	}
 	return res;
 }
@@ -934,17 +1003,17 @@ R_LoadPic(const char *name, byte *pic, int width, int realwidth,
 		{
 			for (j = 0; j < image->width; j++, k++)
 			{
-				scrap_texels[texnum][(y + i) * BLOCK_WIDTH + x + j] = pic[k];
+				scrap_texels[texnum][(y + i) * gl_state.scrap_width + x + j] = pic[k];
 			}
 		}
 
 		image->texnum = TEXNUM_SCRAPS + texnum;
 		image->scrap = true;
 		image->has_alpha = true;
-		image->sl = (x + 0.01) / (float)BLOCK_WIDTH;
-		image->sh = (x + image->width - 0.01) / (float)BLOCK_WIDTH;
-		image->tl = (y + 0.01) / (float)BLOCK_WIDTH;
-		image->th = (y + image->height - 0.01) / (float)BLOCK_WIDTH;
+		image->sl = (float)x / gl_state.scrap_width;
+		image->sh = (float)(x + image->width) / gl_state.scrap_width;
+		image->tl = (float)y / gl_state.scrap_height;
+		image->th = (float)(y + image->height) / gl_state.scrap_height;
 	}
 	else
 	{
