@@ -1131,7 +1131,7 @@ GL3_DrawEntitiesOnList(void)
 	/* draw transparent entities
 	   we could sort these if it ever
 	   becomes a problem... */
-	glDepthMask(0);
+	glDepthMask(GL_FALSE);
 
 	for (i = 0; i < gl3_newrefdef.num_entities; i++)
 	{
@@ -1176,7 +1176,7 @@ GL3_DrawEntitiesOnList(void)
 
 	GL3_DrawAliasShadows();
 
-	glDepthMask(1); /* back to writing */
+	glDepthMask(GL_TRUE); /* back to writing */
 
 }
 
@@ -1997,6 +1997,7 @@ GetRefAPI(refimport_t imp)
 	ri = imp;
 
 	re.api_version = API_VERSION;
+	re.framework_version = GL3_GetSDLVersion();
 
 	re.Init = GL3_Init;
 	re.Shutdown = GL3_Shutdown;
