@@ -22,16 +22,10 @@
  * =======================================================================
  */
 
-#ifdef OGG
-
 #ifndef CL_SOUND_VORBIS_H
- #define CL_SOUND_VORBIS_H
+#define CL_SOUND_VORBIS_H
 
-/* The OGG codec can return the samples in a number
- * of different formats, we use the standard signed
- * short format. */
- #define OGG_SAMPLEWIDTH 2
- #define OGG_DIR "music"
+#include "local.h"
 
 typedef enum
 {
@@ -40,36 +34,15 @@ typedef enum
 	STOP
 } ogg_status_t;
 
-typedef enum
-{
-	ABS,
-	REL
-} ogg_seek_t;
-
+int OGG_Status(void);
+void OGG_InitTrackList(void);
 void OGG_Init(void);
+void OGG_PlayTrack(int trackNo, qboolean cdtrack, qboolean immediate);
+void OGG_RecoverState(void);
+void OGG_SaveState(void);
 void OGG_Shutdown(void);
-void OGG_Reinit(void);
-qboolean OGG_Check(char *name);
-void OGG_Seek(ogg_seek_t type, double offset);
-void OGG_LoadFileList(void);
-void OGG_LoadPlaylist(char *name);
-qboolean OGG_Open(ogg_seek_t type, int offset);
-qboolean OGG_OpenName(char *filename);
-int OGG_Read(void);
-void OGG_Sequence(void);
 void OGG_Stop(void);
 void OGG_Stream(void);
-void S_RawSamplesVol(int samples, int rate, int width,
-		int channels, byte *data, float volume);
-
-/* Console commands. */
-void OGG_ListCmd(void);
-void OGG_ParseCmd(char *arg);
-void OGG_PauseCmd(void);
-void OGG_PlayCmd(void);
-void OGG_ResumeCmd(void);
-void OGG_SeekCmd(void);
-void OGG_StatusCmd(void);
+void OGG_LoadAsWav(char *filename, wavinfo_t *info, void **buffer);
 
 #endif
-#endif
