@@ -34,6 +34,7 @@ FILE *logfile;
 cvar_t *logfile_active;  /* 1 = buffer log, 2 = flush after each print */
 jmp_buf abortframe; /* an ERR_DROP occured, exit the entire frame */
 int server_state;
+cvar_t *color_terminal;
 
 static int rd_target;
 static char *rd_buffer;
@@ -41,7 +42,7 @@ static int rd_buffersize;
 static void (*rd_flush)(int target, char *buffer);
 
 void
-Com_BeginRedirect(int target, char *buffer, int buffersize, void (*flush))
+Com_BeginRedirect(int target, char *buffer, int buffersize, void (*flush)(int, char *))
 {
 	if (!target || !buffer || !buffersize || !flush)
 	{
@@ -82,8 +83,13 @@ Com_VPrintf(int print_level, const char *fmt, va_list argptr)
 	{
 		int i;
 		char msg[MAXPRINTMSG];
+
 		int msgLen = vsnprintf(msg, MAXPRINTMSG, fmt, argptr);
-		if(msgLen >= MAXPRINTMSG)  msgLen = MAXPRINTMSG-1;
+		if (msgLen >= MAXPRINTMSG || msgLen < 0) {
+			msgLen = MAXPRINTMSG-1;
+			msg[msgLen] = '\0';
+		}
+
 		if (rd_target)
 		{
 			if ((msgLen + strlen(rd_buffer)) > (rd_buffersize - 1))
@@ -100,10 +106,22 @@ Com_VPrintf(int print_level, const char *fmt, va_list argptr)
 		Con_Print(msg);
 	#endif
 
+		if ((msg[0] == 0x01 || msg[0] == 0x02) &&
+		    color_terminal && color_terminal->value)
+		{
+			// skip color marker
+			i = 1;
+		}
+		else
+		{
+			i = 0;
+		}
+
 		// remove unprintable characters
-		for(i=0; i<msgLen; ++i)
+		for(; i<msgLen; ++i)
 		{
 			char c = msg[i];
+
 			if(c < ' ' && (c < '\t' || c > '\r'))
 			{
 				switch(c)
@@ -136,7 +154,13 @@ Com_VPrintf(int print_level, const char *fmt, va_list argptr)
 		/* logfile */
 		if (logfile_active && logfile_active->value)
 		{
-			char name[MAX_QPATH];
+			char name[MAX_OSPATH];
+
+			if ((msg[0] == 0x01) || (msg[0] == 0x02))
+			{
+				// remove color marker
+				msg[0] = ' ';
+			}
 
 			if (!logfile)
 			{
@@ -144,12 +168,12 @@ Com_VPrintf(int print_level, const char *fmt, va_list argptr)
 
 				if (logfile_active->value > 2)
 				{
-					logfile = fopen(name, "a");
+					logfile = Q_fopen(name, "a");
 				}
 
 				else
 				{
-					logfile = fopen(name, "w");
+					logfile = Q_fopen(name, "w");
 				}
 			}
 
@@ -171,7 +195,7 @@ Com_VPrintf(int print_level, const char *fmt, va_list argptr)
  * to the apropriate place.
  */
 void
-Com_Printf(char *fmt, ...)
+Com_Printf(const char *fmt, ...)
 {
 	va_list argptr;
 	va_start(argptr, fmt);
@@ -183,7 +207,7 @@ Com_Printf(char *fmt, ...)
  * A Com_Printf that only shows up if the "developer" cvar is set
  */
 void
-Com_DPrintf(char *fmt, ...)
+Com_DPrintf(const char *fmt, ...)
 {
 	va_list argptr;
 	va_start(argptr, fmt);
@@ -196,7 +220,7 @@ Com_DPrintf(char *fmt, ...)
  * cvars is set
  */
 void
-Com_MDPrintf(char *fmt, ...)
+Com_MDPrintf(const char *fmt, ...)
 {
 	va_list argptr;
 	char msg[MAXPRINTMSG];
@@ -218,7 +242,7 @@ Com_MDPrintf(char *fmt, ...)
  * do the apropriate things.
  */
 void
-Com_Error(int code, char *fmt, ...)
+Com_Error(int code, const char *fmt, ...)
 {
 	va_list argptr;
 	static char msg[MAXPRINTMSG];
@@ -243,7 +267,6 @@ Com_Error(int code, char *fmt, ...)
 		recursive = false;
 		longjmp(abortframe, -1);
 	}
-
 	else if (code == ERR_DROP)
 	{
 		Com_Printf("********************\nERROR: %s\n********************\n",
@@ -255,7 +278,6 @@ Com_Error(int code, char *fmt, ...)
 		recursive = false;
 		longjmp(abortframe, -1);
 	}
-
 	else
 	{
 		SV_Shutdown(va("Server fatal crashed: %s\n", msg), false);
