@@ -51,8 +51,9 @@ extern int Q_strlcat ( char * dst , const char * src , int size ) ;
 extern int Q_strlcpy ( char * dst , const char * src , int size ) ;
 extern char * Q_strlwr ( char * s ) ;
 extern void Com_sprintf ( char * dest , int size , char * fmt , ... ) ;
-extern int Q_strcasecmp ( char * s1 , char * s2 ) ;
-extern int Q_strncasecmp ( char * s1 , char * s2 , int n ) ;
+extern int Q_strcasecmp ( const char * s1 , const char * s2 ) ;
+extern int Q_strncasecmp ( const char * s1 , const char * s2 , int n ) ;
+extern char *Q_strcasestr ( const char *s1, const char *s2 ) ;
 extern int Q_stricmp ( const char * s1 , const char * s2 ) ;
 extern void Com_PageInMemory ( byte * buffer , int size ) ;
 extern char * COM_Parse ( char * * data_p ) ;
@@ -723,6 +724,7 @@ extern void SP_target_secret ( edict_t * ent ) ;
 extern void use_target_secret ( edict_t * ent , edict_t * other , edict_t * activator ) ;
 extern void SP_target_help ( edict_t * ent ) ;
 extern void Use_Target_Help ( edict_t * ent , edict_t * other , edict_t * activator ) ;
+extern void Target_Help_Think ( edict_t * ent );
 extern void SP_target_speaker ( edict_t * ent ) ;
 extern void Use_Target_Speaker ( edict_t * ent , edict_t * other , edict_t * activator ) ;
 extern void SP_target_temp_entity ( edict_t * ent ) ;
